'###############################################################################
' BEGIN Phnx_Grid_Class32
'###############################################################################

' Keyboard Interface
' ------------------
' Key                Action
' --------------------------------------------------------------------
' Arrow key           | Moves focus one cell in direction of the arrow
' ENTER               | Moves the focus to the cell in the next row
' SHIFT+ENTER         | Moves the focus to the cell the previous row
' TAB                 | Moves the focus to the cell the next column
' SHIFT+TAB           | Moves the focus to the cell the previous column
' SHIFT+UP            | Extends the selection to the previous row(focus is not changed)
' CTRL+SHIFT+UP       | Extends the selection to the last first row
' SHIFT+DOWN          | Extends the selection to the next row(focus is not changed)
' CTRL+SHIFT+DOWN     | Extends the selection to the last row
' SHIFT+LEFT          | Extends the selection to the previous column(focus is not changed)
' CTRL+SHIFT+LEFT     | Extends the selection to the first column
' SHIFT+RIGHT         | Extends the selection to the next column(focus is not changed)
' CTRL+SHIFT+RIGHT    | Extends the selection to the last column
' HOME                | Moves the focus to the first column in the current row
' SHIFT+HOME          | Extends the selection to the first column
' CTRL+HOME           | Moves the focus to the first column and first row
' CTRL+SHIFT+HOME     | Extends the selection to the first column and the first row
' END                 | Moves the focus to the last column in the current row
' SHIFT+END           | Extends the selection to the last column
' CTRL+END            | Moves the focus to the last column and the last row
' CTRL+SHIFT+END      | Extends the selection to the last column and the last row
' PAGE UP             | Moves the focus up one screen
' SHIFT+PAGE UP       | Extends the selection up one screen
' ALT+PAGE UP         | Moves the focus left one screen
' ALT+SHIFT+PAGE UP   | Extends the focus left one screen
' PAGE DOWN           | Moves the focus down one screen
' SHIFT+PAGE DOWN     | Extends the focus down one screen
' ALT+PAGE DOWN       | Moves the focus right one screen
' ALT+SHIFT+PAGE DOWN | Extends the focus right one screen

' left button                 | Drag move selection
' CTRL+left button            | Drag copy selection
' CTRL+SHIFT+left button      | Drag move insert selection
' CTRL+ALT+SHIFT+left button  | Drag copy insert selection

' Cell types
%GCT_GENERAL      = &H0?
%GCT_CHECKBUTTON  = &H1?
%GCT_PUSHBUTTON   = &H2?
%GCT_FLATBUTTON   = &H3?
%GCT_CHECKBOX     = &H4?
%GCT_RADIO        = &H5?
%GCT_LIST         = &H6?
%GCT_EDITLIST     = &H7?
%GCT_URL          = &H8?
'%GCT_OWNERDRAW    = &H9?
%GCT_BUTTON       = &HA?
%GCT_EDITBUTTON   = &HB?

' Grid item masks (dwMask member of structures)
%GIF_STATE       = &H1         ' the dwStateMask and dwState members must be filled in
%GIF_FONT        = &H2         ' the tlf member must be filled in
%GIF_TEXT        = &H4         ' the pszText and cchTextMax members must be filled in
%GIF_ALIGN       = &H8         ' the bAlign, bImageOnRight, bClip, and bCenterAcross members must be filled in
%GIF_IMAGE       = &H10        ' the iImage member must be filled in
%GIF_STATEIMAGE  = &H20        ' the iImageState member must be filled
%GIF_BKGNDIMAGE  = &H40        ' the hInstance, pszImage, cbEndcoded, bImageType, and bStretch members must be filled in
%GIF_LPARAM      = &H80        ' the lParam member must be filled in
%GIF_COLOR       = &H100       ' the dwTextColor and dwBackColor members must be filled in
%GIF_BORDER      = &H200       ' the bBorderStyle, bPenStyle, dwPenColor and bPenWidth members must be filled in
%GIF_CELLTYPE    = &H400       ' the bCellType, bOwnerDraw, bMasked, bIntelliSense, bNode and iLast members must be filled in
%GIF_INDENT      = &H800       ' the bIndentIntegral member must be filled in
%GIF_SIZE        = &H1000      ' the wSize member must be filled in
%GIF_MERGER      = &H2000      ' this flag cannot be used to merge or unmerge cells. Its is use as a read only flag with GCM_GETITEM, or to initialize a grid (GCM_SETITEM) with predefined data.
%GIF_FORMAT      = &H4000
%GIF_BITMAP      = %GIF_BKGNDIMAGE
%GIF_ALL         = %GIF_STATE OR %GIF_FONT OR %GIF_TEXT OR %GIF_ALIGN OR _
                   %GIF_IMAGE OR %GIF_STATEIMAGE OR %GIF_BKGNDIMAGE OR %GIF_LPARAM OR _
                   %GIF_COLOR OR %GIF_BORDER OR %GIF_CELLTYPE OR %GIF_INDENT OR _
                   %GIF_SIZE OR %GIF_MERGER OR %GIF_FORMAT

' Grid item states
%GIS_CHECKED      = &H00000001???
%GIS_SELECTED     = &H00000002???
%GIS_READONLY     = &H00000004???
%GIS_LOCKED       = &H00000008???   ' the grid sends GCN_ITEMCLICK notifications
%GIS_NOLABELEDIT  = &H00000010???
%GIS_DISABLED     = &H00000020???   ' the grid does not send GCN_ITEMCLICK notifications
%GIS_MODIFIED     = &H00000040???
%GIS_HOTTRACK     = &H00000080???
%GIS_PUSHED       = &H00000100???
%GIS_HIGHLIGHTED  = &H00000200???
%GIS_SHADOWED     = &H00000400???
%GIS_FROZEN       = &H00000800???   ' the cell is locked and the grid does not send GCN_ITEMCLICK notifications
%GIS_EXPANDED     = &H00001000???
%GIS_ONDEMAND     = &H00002000???
%GIS_PINNEDLEFT   = &H00004000???   ' the column is pinned to the left(does not scroll)
%GIS_PINNEDRIGHT  = &H00008000???   ' the column is pinned to the right(does not scroll)
%GIS_PINNEDTOP    = &H00010000???   ' the row is pinned to the top(does not scroll)
%GIS_PINNEDBOTTOM = &H00020000???   ' the row is pinned to the bottom(does not scroll)

%GIS_ALL          = %GIS_CHECKED OR %GIS_SELECTED OR %GIS_READONLY OR %GIS_LOCKED OR %GIS_NOLABELEDIT OR _
                    %GIS_DISABLED OR %GIS_MODIFIED OR %GIS_HOTTRACK OR %GIS_PUSHED OR %GIS_HIGHLIGHTED OR _
                    %GIS_SHADOWED OR %GIS_FROZEN OR %GIS_EXPANDED OR %GIS_ONDEMAND OR %GIS_PINNEDLEFT OR _
                    %GIS_PINNEDRIGHT OR %GIS_PINNEDTOP OR %GIS_PINNEDBOTTOM


' Hit test values
' Object type(lowrd).  Combined with one of the values below
%GHT_CAPTION    = &H0001??
%GHT_SELALLBTN  = &H0002??
%GHT_COLHEADER  = &H0004??
%GHT_ROWHEADER  = &H0008??
%GHT_CELL       = &H0010??
%GHT_NOOBJECT   = &HFFFF??
' Location (hiwrd).
%GHT_LABEL      = &H0001??          ' label
%GHT_IMAGE      = &H0002??          ' normal image
%GHT_STATEIMAGE = &H0004??          ' state image
%GHT_LDIVIDER   = &H0008??          ' left divider (in header)
%GHT_RDIVIDER   = &H0010??          ' right divider (in header)
%GHT_LMARGIN    = %GHT_LDIVIDER     ' left margin (in cell)
%GHT_RMARGIN    = %GHT_RDIVIDER     ' right margin (in cell)
%GHT_TMARGIN    = &H0040??          ' top margin (in cell)
%GHT_BMARGIN    = &H0080??          ' bottom margin (in cell)
%GHT_NODEBUTTON = &H0100??          ' node button
%GHT_NOWHERE    = &HFFFF??
%GHT_BACKGROUND = %GHT_NOWHERE

' Imagelist types
%GISIL_CELL      = 0
%GISIL_STATE     = 1
%GISIL_HEADER    = 2

' Cell border index
%GCBI_LEFT        = 0
%GCBI_TOP         = 1
%GCBI_RIGHT       = 2
%GCBI_BOTTOM      = 3
%GCBI_OUTLINE     = 4       ' used for outlining selections

' Cell border pen style
%GCBP_SOLID       = 0
%GCBP_DASH        = 1
%GCBP_DOT         = 2
%GCBP_DASHDOT     = 3
%GCBP_DASHDOTDOT  = 4
%GCBP_NULL        = 5
%GCBP_DOUBLE      = 6

' Background image types
%GIMAGE_BMP           = 0     ' Bmp  - (IMAGE_BITMAP     ) Windows Bitmap Format
%GIMAGE_ICO           = 1     ' Ico  - (IMAGE_ICON       ) Icon Format
%GIMAGE_CUR           = 2     ' Cur  - (IMAGE_CURSOR     ) Icon Format
%GIMAGE_EMF           = 3     ' Emf  - (IMAGE_ENHMETAFILE) Enhanced Metafile
%GIMAGE_WMF           = 4     ' Wmf  - Windows Metafile
%GIMAGE_PNG           = 5     ' Png  - ortable Network Graphics
%GIMAGE_JPG           = 6     ' Jpeg - Joint Photographic Experts Group
%GIMAGE_TIF           = 7     ' Tiff - Tagged Image File Format
%GIMAGE_GIF           = 8     ' Gif  - Graphics Interchange Format
%GIMAGE_WMP           = 9     ' HDP  - Windows Media Photo
%GIMAGE_OTHER         = 254
%GIMAGE_AUTO          = 255   ' Auto detect

TYPE GRIDIMAGEITEM
  #IF %DEF(%UNICODE)
  pszImage    AS WSTRINGZ PTR ' fully qualified path and filename or resource id or offset in xml file of background image
  #ELSE
  pszImage    AS ASCIIZ PTR   ' fully qualified path and filename or resource id or offset in xml file of background image
  #ENDIF
  cbEncoded   AS LONG         ' size of Base64 endcoding data for image(nonzero means the image is loaded from a format file)
  uImageType  AS DWORD        ' image types(bmp, ico, cur, emf, wmf, png, jpeg, tiff, gif, hdp)
  rImageKey   AS GUID
  hInstance   AS DWORD
END TYPE

TYPE GRIDIMAGECOLLECTION  ' gic
  lpImages    AS DWORD        ' array of GRIDIMAGEITEM structs
  cImages     AS LONG         ' count of images in lpImages
  lpEncoded   AS DWORD        ' address of encoded images
  cbEncoded   AS LONG         ' count of bytes in encoded data
END TYPE

TYPE GRIDIMAGELIST    ' gil
  hIml        AS DWORD        ' ImageList handle
  uFlags      AS DWORD        ' ILC_COLOR24, ILC_COLOR32
  cxImage     AS WORD         ' width of images in ImageList
  cyImage     AS WORD         ' height of images in ImageList
  uImageType  AS DWORD        ' image types(bmp, ico, cur)
  dwMaskColor AS DWORD        ' mask color
  tgic        AS GRIDIMAGECOLLECTION
END TYPE

TYPE GRIDIMAGESIZE    ' gis
  uImageType  AS DWORD        ' image types(bmp, ico, cur, emf, wmf, png, jpeg, tiff, gif, hdp)
  cxImage     AS long         ' width of images in ImageList
  cyImage     AS LONG         ' height of images in ImageList
END TYPE

' Image position relative to text in cell
%GCIL_LEFT        = 0
%GCIL_RIGHT       = 1
%GCIL_TOP         = 2   ' not supported for column and row headers
%GCIL_BOTTOM      = 3   ' not supported for column and row headers

' Source of font used object
%FONT_DEFAULT     = &H00?
%FONT_USER        = &H01?

' Default color flag
%COLOR_DEFAULT    = &HFFFFFFFF???

' Maximum size of tooltip and text callback buffers
%GRIDTIPSIZE      = 2048
%LABELEDITSIZE    = 8192

' Line ending bytes.  Examples -
' byte sequence byte1 byte2 count
' <cf><lf>      &H0D  &H0A  2
' <cr>          &H0D  &H0D  1
' <cr><cr>      &H0D  &H0D  2

#IF %DEF(%UNICODE)
TYPE EOLBYTES         ' eol
  bByte1  AS WORD     ' First word in end of line sequence
  bByte2  AS WORD     ' Second word in end of line sequence
  bByte3  AS WORD     ' Third word in end of line sequence
  bCount  AS BYTE     ' Count of words in EOL sequence
END TYPE
#ELSE
TYPE EOLBYTES         ' eol
  bByte1  AS BYTE     ' First byte in end of line sequence
  bByte2  AS BYTE     ' Second byte in end of line sequence
  bByte3  AS BYTE     ' Third byte in end of line sequence
  bCount  AS BYTE     ' Count of bytes in EOL sequence
END TYPE
#EndIf

' Used to defined a range of items
' This has the same layout as the RECT structure and can be
' used in place of the RECT structure to defined a range.
TYPE GRIDRANGE    ' gr
  iLeft     AS LONG
  iTop      AS LONG
  iRight    AS LONG
  iBottom   AS LONG
END TYPE

' The I_GRIDNEEDITEMS flag causes a parent item to request child items once on expanding.
' To allow the parent to always request child items on expanding, set its state to GIS_ONDEMAND.
%I_GRIDNEEDITEMS  = -1

' Note: to retrieve the size of the buffer, minus the terminating null, that
'       is needed for the text of an item, set the pszText member to Null.

TYPE GRIDITEM   ' gi
  wSize                 AS WORD         ' width/height of item
  dwHitCode             AS DWORD        ' type of item
  dwMask                AS DWORD        ' attributes of item being requested or set
  dwStateMask           AS DWORD        ' state of item being requested or set
  dwState               AS DWORD        ' state of item
  #IF %DEF(%UNICODE)
  pszText               AS WSTRINGZ PTR ' item text
  #ELSE
  pszText               AS ASCIIZ PTR   ' item text
  #ENDIF
  cchTextMax            AS LONG         ' maximum size of buffer receiving text
  #IF %DEF(%UNICODE)
  pszFormat             AS WSTRINGZ PTR ' item mask
  #ELSE
  pszFormat             AS ASCIIZ PTR   ' item mask
  #ENDIF
  cchFormatMax          AS LONG         ' maximum size of buffer receiving mask
  bClipLiterals         AS BYTE         ' mask clip literals flag
  iImage                AS LONG         ' index of image in normal imagelist(-1 = no image)
  iImageState           AS LONG         ' index of state image in state imagelist(-1 = no image)
  #IF %DEF(%UNICODE)
  pszImage              AS WSTRINGZ PTR ' fully qualified path and filename or resource id or offset in xml file of background image
  #ELSE
  pszImage              AS ASCIIZ PTR   ' fully qualified path and filename or resource id or offset in xml file of background image
  #ENDIF
  cbEncoded             AS LONG         ' size of Base64 endcoding data for image(nonzero means the image is loaded from a format file)
  bImageType            AS BYTE         ' image type of background image(bmp, ico, cur, emf, wmf, png, jpeg, tiff, gif, hdp)
  rImageKey             AS GUID         ' Identifier of image in image collection
  bImageMargin          AS BYTE         ' image margin(15 pixels maximum)
  bStretch              AS BYTE         ' stretch background image flag
  bIndentIntegral       AS INTEGER      ' number of increments of the state image width by which text/image is indented
  dwAlign               AS DWORD        ' text alignment
  bClip                 AS BYTE         ' clipping flag
  bCenterAcross         AS BYTE         ' center across selection flag
  bImageOnRight         AS BYTE         ' image right of text flag
  bFontSrc              AS BYTE         ' source of font flag
  tlf                   AS LOGFONT      ' font attributes
  dwTextColor           AS DWORD        ' text color
  dwBackColor           AS DWORD        ' background color
  bCellType             AS BYTE         ' cell type of item
  bOwnerDraw            AS BYTE         ' nonzero if cell is owner-drawn
  bMasked               AS BYTE         ' nonzero if cell is masked
  bIntelliSense         AS BYTE         ' nonzero if cell supports IntelliSense
  bLabelEditSort        AS BYTE         ' nonzero if cell sorts drop-down list
  bOwnerDrawList        AS BYTE         ' nonzero if owner of grid is responsible for drawing drop-list items
  bLabelEditEndOnClick  AS BYTE         ' nonzero if clicking a drop-down list item ends label-editing
  bNode                 AS BYTE         ' nonzero if cell is a node
  bInside               AS BYTE          ' If FALSE, rectangular selection is oulined with the new border
  bBorderStyle(4)       AS BYTE         ' draw border flag. If TRUE, a border is drawn on the associated edge
  bPenStyle(4)          AS BYTE         ' style of pen used to draw border
  dwPenColor(4)         AS DWORD        ' color of pen used to draw border
  bPenWidth(4)          AS BYTE         ' width of pen used to draw border
  bBorderEdit(4)        AS BYTE         ' border modified flag
  bMerge                AS BYTE         ' If TRUE, cell is part of a merger
  iLeft                 AS LONG         ' column index of cell in upper left corner of merger
  iTop                  AS LONG         ' row index of cell in upper left corner of merger
  iRight                AS LONG         ' column index of cell in bottom right corner of merger
  iBottom               AS LONG         ' row index of cell in bottom right corner of merger
  iParent               AS LONG         ' readonly zero-based index of parent.
  iLast                 AS LONG         ' Flag that indicates whether the item has associated child items.
                                        ' I_TREENEEDITEMS = the control requests child items to be inserted on demand.
                                        '                   If the control has the TRS_HASBUTTONS style, it displays a button
                                        '                   indicating the presence of child items.
                                        ' 0               = the item has no children.
                                        ' > 0             = zero-based index of last child.
  lParam                AS LONG         ' application-defined value
END TYPE

TYPE GRIDITEMEX         ' gdix
  iLeft           AS LONG
  iTop            AS LONG
  iRight          AS LONG
  iBottom         AS LONG
  hInstance       AS DWORD            ' handle of module containing background images
  tgi             AS GRIDITEM
END TYPE

' Used by the GCM_HITTEST message
TYPE GRIDHITTEST   ' ght
  tpt             AS POINTAPI       ' IN: location to test in client coordinates
  iCol            AS LONG           ' OUT: zero-based column index of item
  iRow            AS LONG           ' OUT: zero-based row index of item
  trcBnd          AS RECT           ' OUT: bounding rectangle of item
  dwHitCode       AS DWORD          ' OUT: object type and location (hit-test value)
END TYPE

' Used by the GCM_SETHEADERIDEALSIZE, GCM_GETHEADERIDEALSIZE messages
TYPE GRIDHEADERSIZE 'ghs
  iStart          AS LONG
  iEnd            AS LONG
  dwHitCode       AS DWORD        ' GHT_CAPTION, GHT_COLHEADER, or GHT_ROWHEADER
  pwWidth         AS WORD PTR     ' address of variable used to receive width or set to TRUE if setting width
  pwHeight        AS WORD PTR     ' address of variable used to receive height or set to TRUE if setting height
  fExclHidden     AS LONG         ' True = exclude hidden items
END TYPE

' The action flag for the GCM_EXPAND message and the GCN_ITEMEXPANDED
' and GCN_ITEMEXPANDING notifications, it can be one of the
' following values:
%GIE_COLLAPSE         = 1
%GIE_EXPAND           = 2
%GIE_TOGGLE           = 3
%GIE_EXPANDSUBTREES   = 4
%GIE_COLLAPSESUBTREES = 5

' This structure is used by the GCM_EXPAND message
TYPE GRIDEXPAND     ' ge
  iCol            AS LONG
  iRow            AS LONG
  dwAction        AS DWORD
END TYPE

' This structure is used by the GCN_ITEMEXPANDING, GCN_ITEMEXPANDED notifications
TYPE GRIDITEMEXPAND     ' gie
  hdr             AS NMHDR
  iCol            AS LONG
  iRow            AS LONG
  dwAction        AS DWORD
  tgi             AS GRIDITEM
END TYPE

' This structure is used by the GCM_SETITEMPARENT message
TYPE GRIDITEMPARENT  ' gip
  iCol            AS LONG           ' zero-based column index of cells
  iParent         AS LONG           ' zero-based row index of parent cell
  iTop            AS LONG           ' zero-based row index of first cell
  iBottom         AS LONG           ' zero-based row index of last cell
END TYPE

' The action flag(wParam) of the GCM_GETNEXTITEM message.
%GGNI_ROOT                  = &H00000000??? ' The very first item of the tree containing the reference cell
%GGNI_NEXT                  = &H00000001??? ' The next item that is a sibling of the item specified by the reference cell
%GGNI_PREVIOUS              = &H00000002??? ' The previous item that is a sibling of the item specified by the reference cell
%GGNI_PARENT                = &H00000003??? ' The parent of the item specified by the reference cell
%GGNI_CHILD                 = &H00000004??? ' The first child item of the item specified by the reference cell
%GGNI_NEXTVISIBLE           = &H00000005??? ' The next visible item that follows the specified reference cell
%GGNI_PREVIOUSVISIBLE       = &H00000006??? ' The previous visible item that follows the specified reference cell
%GGNI_CARET                 = &H00000007??? ' The currently selected(focus) cell
%GGNI_LASTCHILD             = &H00000008??? ' The last child item of the item specified by the reference cell
%GGNI_NEXTPINNEDLEFT        = &H00000009??? ' The next column pinned-left that follows the specified reference column
%GGNI_PREVIOUSPINNEDLEFT    = &H0000000A??? ' The previous column pinned-left that follows the specified reference column
%GGNI_NEXTPINNEDRIGHT       = &H0000000B??? ' The next column pinned-right that follows the specified reference column
%GGNI_PREVIOUSPINNEDRIGHT   = &H0000000C??? ' The previous column pinned-right that follows the specified reference column
%GGNI_NEXTPINNEDTOP         = &H0000000D??? ' The next row pinned-top that follows the specified reference row
%GGNI_PREVIOUSPINNEDTOP     = &H0000000E??? ' The previous row pinned-top that follows the specified reference row
%GGNI_NEXTPINNEDBOTTOM      = &H0000000F??? ' The next row pinned-bottom that follows the specified reference row
%GGNI_PREVIOUSPINNEDBOTTOM  = &H00000010??? ' The previous row pinned-bottom that follows the specified reference row

' This structure is used by the GCM_GETNEXTITEM and GCM_ISITEMVISIBLE messages
TYPE GRIDCELL  ' gc
  dwHitCode       AS DWORD          ' type of item
  iCol            AS LONG           ' zero-based column index of cell
  iRow            AS LONG           ' zero-based row index of cell
END TYPE

' GRIDLABELEDIT fCancelled member values
%GLE_ESCAPE_KEY       = 1           ' Esc key was pressed
%GLE_BUTTON_CLICKED   = 2           ' Button(GCT_BUTTON, GCT_EDITBUTTON) was clicked
%GLE_LISTITEM_CLICKED = 3           ' Item selected by clicking(GCT_LIST, GCT_EDITLIST)

' Used by the GCN_BEGINLABELEDIT, GCN_ENDLABELEDIT notifications
TYPE GRIDLABELEDIT  ' gle
  hdr             AS NMHDR
  iCol            AS LONG
  iRow            AS LONG
  hWndEdit        AS DWORD        ' handle of label edit control
  fCancelled      AS LONG
  tgi             AS GRIDITEM
END TYPE

' Used by the GCN_MEASURELISTITEM notification
TYPE GRIDMEASURELISTITEM   ' gmli
  hdr             AS NMHDR
  iCol            AS LONG           ' zero-based column index of cell
  iRow            AS LONG           ' zero-based row index of cell
  hWndEdit        AS DWORD          ' handle of label-editing control
  hWndList        AS DWORD          ' handle of label-editing drop-list window
  wHeight         AS WORD           ' height of items in the drop-list
END TYPE

' GRIDDRAWLISTITEM dwAction field flags:
%GDLIA_DRAWENTIRE   = &H1       ' The entire drop-list needs to be drawn.
%GDLIA_SELECT       = &H2       ' The drop-list has lost or gained the keyboard focus. The dwState member should be checked to determine whether the control has the focus.
%GDLIA_FOCUS        = &H4       ' The selection status has changed. The dwState member should be checked to determine the new selection state.

' GRIDDRAWLISTITEM dwState field flags:
%GDLIS_SELECTED     = &H0001    ' The item is selected.
%GDLIS_FOCUS        = &H0010    ' The item has the keyboard focus.
%GDLIS_HOTLIGHT     = &H0040    ' The item is being hot-tracked.

' Used by the GCN_DRAWLISTITEM notification
TYPE GRIDDRAWLISTITEM   ' gdli
  hdr             AS NMHDR
  iCol            AS LONG           ' zero-based column index of cell
  iRow            AS LONG           ' zero-based row index of cell
  hWndEdit        AS DWORD          ' handle of label-editing control
  hWndList        AS DWORD          ' handle of label-editing drop-list window
  hDC             AS DWORD          ' device context to use when performing drawing operations
  trc             AS RECT           ' bounding rectangle(clipping rectangle) of drop-list item
  iItem           AS LONG           ' zero-based index of drop-list item
  lItemData       AS LONG           ' application-defined value associated with drop-list item
  dwAction        AS DWORD
  dwState         AS DWORD
  tgi             AS GRIDITEM
END TYPE

' Used by the GCN_ONCHARLABELEDIT notification
TYPE GRIDONCHAR   ' goc
  hdr             AS NMHDR
  iCol            AS LONG
  iRow            AS LONG
  hWndEdit        AS DWORD        ' handle of label-editing control
  hWndList        AS DWORD        ' handle of label-editing drop-list window
  #IF %DEF(%UNICODE)
  pszCompareText  AS WSTRINGZ PTR ' address of text to compare(start of text to caret location)
  #ELSE
  pszCompareText  AS ASCIIZ PTR   ' address of text to compare(start of text to caret location)
  #ENDIF
  cchCompareText  AS LONG         ' count of characters in lpCompareText or minus one if text is null-terminated
  tgi             AS GRIDITEM
END TYPE

' Used by the GCN_VALIDATIONERROR notification
TYPE GRIDVALIDATIONERROR  ' gve
  hdr             AS NMHDR
  iCol            AS LONG
  iRow            AS LONG
  hWndEdit        AS DWORD        ' handle of label edit control
  lStartPos       AS LONG         ' zero-based index of the first invalid character
  #IF %DEF(%UNICODE)
  pszInvalidText  AS WSTRINGZ PTR ' address of invalid text
  #ELSE
  pszInvalidText  AS ASCIIZ PTR   ' address of invalid text
  #ENDIF
  tgi             AS GRIDITEM
END TYPE

' Used by the GCN_MAXTEXTERROR notification
TYPE GRIDMAXTEXTERROR  ' gmte
  hdr             AS NMHDR
  iCol            AS LONG
  iRow            AS LONG
  hWndEdit        AS DWORD        ' handle of label edit control
  tgi             AS GRIDITEM
END TYPE

' Used by the GCN_ITEMCHANGED, GCN_GETDISPINFO, and GCN_GETINFOTIP,
' GCN_NEEDITEMS, GCN_GETBUSYINFO
' notification messages.
' GCN_ITEMCHANGED:
' the iLeft, iTop, iRight, and iBottom members specify the relevant items.
' The dwMask member specifies the attributes that have changed.
' GCN_GETDISPINFO:
' the iLeft, iTop, members specify the relevant item.
' if the hInstance member is null, the pszImage member of the GRIDITEM
' structure is set to the filename of the image to load.  If it is
' non-null, the pszImage member is set to the identifier of a resource
' image in the module referenced by the hInstance member.
' GCN_GETINFOTIP:
' the iLeft, iTop, members specify the relevant item.
' if hInstance is null, the buffer pointed to by the pszText member
' of the GRIDITEM structure is used to set the item's informative
' text.  This buffer has a maximum size of GRIDTIPSIZE. If it is
' non-null, the pszText member is set to the identifier of a resource
' string in the module referenced by the hInstance member.
' GCN_GETBUSYINFO:
' the iLeft, iTop, members specify the relevant item.
' if hInstance is null, the buffer pointed to by the pszText member
' of the GRIDITEM structure is used to set the item's informative
' text.  This buffer has a maximum size of GRIDTIPSIZE. If it is
' non-null, the pszText member is set to the identifier of a resource
' string in the module referenced by the hInstance member.
' GCN_NEEDITEMS:
' the iLeft, iTop members specify the relevant items.
TYPE GRIDDISPINFO     ' gdi
  hdr             AS NMHDR
  iLeft           AS LONG
  iTop            AS LONG
  iRight          AS LONG
  iBottom         AS LONG
  hInstance       AS DWORD            ' instance of module containing string or image resource
  tgi             AS GRIDITEM
END TYPE

' key flags stored in wKeyFlags
%GKF_ALT       = &H0001??
%GKF_CONTROL   = &H0002??
%GKF_SHIFT     = &H0004??

TYPE GRIDITEMACTIVATE   ' gia
  hdr             AS NMHDR
  iCol            AS LONG           ' zero-based column index of item that was clicked
  iRow            AS LONG           ' zero-based row index of item that was clicked
  tpt             AS POINTAPI       ' location where the event occurred in client coordinates
  wKeyFlags       AS WORD           ' flags specifying the modifier keys that were pressed at the time of the activation
  tgi             AS GRIDITEM
END TYPE

TYPE GRIDKEYDOWN      ' gkd
  hdr             AS NMHDR
  wVirtKey        AS WORD
  wKeyFlags       AS WORD           ' flags specifying the modifier keys that were pressed at the time of the activation
END TYPE

' GCN_ITEMDRAW return values
%GID_DODEFAULT    = 0
%GID_NEWCOLORS    = 1
%GID_SKIPDEFAULT  = 2

' This structure is used by the GCN_ITEMDRAW notification message
TYPE GRIDITEMDRAW       ' gid
  hdr             AS NMHDR
  iCol            AS LONG           ' zero-based column index of item to be drawn
  iRow            AS LONG           ' zero-based column index of item to be drawn
  hInstance       AS DWORD          ' handle of module containing background images
  hDC             AS DWORD          ' device context to use when performing drawing operations
  trc             AS RECT           ' bounding (clipping) rectangle of item
  tgi             AS GRIDITEM
END TYPE

' Used by the GCN_HOTITEMCHANGE notification message
TYPE GRIDHOTITEM    ' ghi
  hdr             AS NMHDR
  iCol            AS LONG           ' zero-based column index of item
  iRow            AS LONG           ' zero-based column index of item
  fLeaving        AS LONG           ' TRUE if mouse is leaving the item
  tgi             AS GRIDITEM
END TYPE

' Used by the GCN_SETFOCUS/GCN_KILLFOCUS notification messages
TYPE GRIDCONTROLFOCUS   ' gcf
  hdr             AS NMHDR
  hWndFocus       AS DWORD          ' handle of window gaining or losing the input focus
END TYPE

' Used by the GCN_FOCUSCHANGING/GCN_FOCUSCHANGED notification messages
TYPE GRIDITEMFOCUS      ' gif
  hdr             AS NMHDR
  iColOld         AS LONG           ' zero-based column index of old item
  iRowOld         AS LONG           ' zero-based column index of old item
  iColNew         AS LONG           ' zero-based column index of new item
  iRowNew         AS LONG           ' zero-based column index of new item
END TYPE

' This structure(deprecated) is used by the GCM_SETITEMTEXT message
' IF wParam is null, the defaults are used.
TYPE GRIDSETTEXT  ' gst
  iCol            AS LONG           ' zero-based column index of start of write operation
  iRow            AS LONG           ' zero-based row index of start of write operation
  fInsert         AS LONG           ' if TRUE, text is inserted.  If FALSE, existing text is overwritten
  fLine1IsHeaders AS LONG           ' if TRUE, the first line in the text defines the column headers
  #IF %DEF(%UNICODE)
  bDelimiter      AS WORD           ' character delimiting strings. If zero, double quote (") is used
  bColSeparator   AS WORD           ' character separating fields. Set to zero to autodetect
  #ELSE
  bDelimiter      AS BYTE           ' character delimiting strings. If zero, double quote (") is used
  bColSeparator   AS BYTE           ' character separating fields. Set to zero to autodetect
  #ENDIF
  teol            AS EOLBYTES       ' bytes defining the end of a row. The default is CRLF.
  fSetProperty    AS LONG           ' if TRUE, the GRIDITEM structure is used to set the attributes of the cells
  tgi             AS GRIDITEM       ' structure defining cell attributes
END TYPE

' This structure(deprecated) is used by the GCM_GETTEXT message
TYPE GRIDGETTEXT  ' ggt
  iLeft           AS LONG           ' zero-based column index of start of write operation
  iTop            AS LONG           ' zero-based row index of start of write operation
  iRight          AS LONG
  iBottom         AS LONG
  fCopyHeaders    AS LONG           ' if TRUE, the first line in the text defines the column headers
  #IF %DEF(%UNICODE)
  bDelimiter      AS WORD           ' character delimiting strings. If zero, double quote (") is used
  bColSeparator   AS WORD           ' character separating fields. If zero, comma (,) is used
  #ELSE
  bDelimiter      AS BYTE           ' character delimiting strings. If zero, double quote (") is used
  bColSeparator   AS BYTE           ' character separating fields. If zero, comma (,) is used
  #ENDIF
  teol            AS EOLBYTES       ' bytes defining the end of a row. The default is CRLF.
END TYPE

' GRIDSETTEXTEX field flags
'
' GSTF_AUTODETECT
' Fields are autodetected using the comma or tab as the column separator
' and crlf as the field separator.  The string delimiter is assumed to be
' the double quote character(") if none specified via the bDelimiter member
' of the GRIDSETTEXTEX structure.
%GSTF_AUTODETECT      = 0
'
' GSTF_USEDEFAULT
' Fields are detected using field metrics set by the GCM_SETFIELDLMETRIC message
%GSTF_USEDEFAULT      = 1
'
' GSTF_SPECIFIED
' Fields are detected using field metrics specified in the GRIDSETTEXTEX structure
%GSTF_SPECIFIED       = 2

' This structure is used by the GCM_SETITEMTEXTEX, GCM_LOADTEXTFILEEX messages
' Note:if the value of dwFlags is GSTF_AUTODETECT, the field metrics members
'      are set to the detected values when GCM_LOADTEXTFILEEX returns.
TYPE GRIDSETTEXTEX  ' gst
  iCol              AS LONG           ' zero-based column index of start of write operation
  iRow              AS LONG           ' zero-based row index of start of write operation
  fInsert           AS LONG           ' if TRUE, text is inserted.  If FALSE, existing text is overwritten
  fLineOneIsHeaders AS LONG           ' if TRUE, the first line in the text defines the column headers
  dwFlags           AS DWORD          ' specifies how fields are detected
  #IF %DEF(%UNICODE)
  bDelimiter        AS WORD           ' character delimiting strings. If zero, double quote (") is used
  bColSeparator     AS WORD           ' character separating fields. Set to zero to autodetect
  #ELSE
  bDelimiter        AS BYTE           ' character delimiting strings. If zero, double quote (") is used
  bColSeparator     AS BYTE           ' character separating fields. Set to zero to autodetect
  #ENDIF
  teol              AS EOLBYTES       ' bytes defining the end of a row. The default is CRLF.
  fSetFieldMetrics  AS LONG           ' If TRUE, the auto-detected field metrics are set as the defaults
  fAutoSizeCols     AS LONG           ' if TRUE, columns are set to their ideal size
  fSetProperty      AS LONG           ' if TRUE, the GRIDITEM structure is used to set the attributes of the cells
  tgi               AS GRIDITEM       ' structure defining cell attributes
  lpText            AS DWORD          ' address of text or fully qualified path and filename
  cchText           AS DWORD          ' length of text. Can be set to zero if text is nul-terminated
END TYPE

' This structure is used by the GCM_GETITEMTEXTEX and GCM_SAVETEXTFILEEX messages
TYPE GRIDGETTEXTEX  ' ggt
  iLeft           AS LONG           ' zero-based column index of start of write operation
  iTop            AS LONG           ' zero-based row index of start of write operation
  iRight          AS LONG
  iBottom         AS LONG
  fCopyHeaders    AS LONG           ' if TRUE, the first line in the text defines the column headers
  dwFlags         AS DWORD          ' specifies how fields are detected
  #IF %DEF(%UNICODE)
  bDelimiter      AS word           ' character delimiting strings. If zero, double quote (") is used
  bColSeparator   AS WORD           ' character separating fields. If zero, comma (,) is used
  #ELSE
  bDelimiter      AS BYTE           ' character delimiting strings. If zero, double quote (") is used
  bColSeparator   AS BYTE           ' character separating fields. If zero, comma (,) is used
  #ENDIF
  teol            AS EOLBYTES       ' bytes defining the end of a row. The default is CRLF.
END TYPE

' This structure is used by the GCM_PASTESPECIAL message
' Note:if the value of dwFlags is GSTF_AUTODETECT, the field metrics members
'      are set to the detected values when GCM_PASTESPECIAL returns.
TYPE GRIDPASTE      ' gp
  iCol              AS LONG           ' zero-based column index of start of write operation
  iRow              AS LONG           ' zero-based row index of start of write operation
  fInsert           AS LONG           ' if TRUE, text is inserted.  If FALSE, existing text is overwritten
  fLineOneIsHeaders AS LONG           ' if TRUE, the first line in the text defines the column headers
  dwFlags           AS DWORD          ' specifies how fields are detected
  #IF %DEF(%UNICODE)
  bDelimiter        AS WORD           ' character delimiting strings. If zero, double quote (") is used
  bColSeparator     AS WORD           ' character separating fields. Set to zero to autodetect
  #ELSE
  bDelimiter        AS BYTE           ' character delimiting strings. If zero, double quote (") is used
  bColSeparator     AS BYTE           ' character separating fields. Set to zero to autodetect
  #ENDIF
  teol              AS EOLBYTES       ' bytes defining the end of a row. The default is CRLF.
  fSetFieldMetrics  AS LONG           ' If TRUE, the auto-detected field metrics are set as the defaults
  fPlainText        AS LONG           ' if TRUE, paste plain text
  lpText            AS DWORD          ' address of text or fully qualified path and filename
  cchText           AS DWORD          ' length of text. Can be set to zero if text is nul-terminated
END TYPE

' This structure is used by the GCM_GETFIELDMETRIC/GCM_SETFIELDMETRIC message
TYPE GRIDFIELDMETRIC    ' gfm
  #IF %DEF(%UNICODE)
  bDelimiter      AS WORD           ' character delimiting strings. If zero, double quote (") is used
  bColSeparator   AS WORD           ' character separating fields. If zero, comma (,) is used
  #ELSE
  bDelimiter      AS BYTE           ' character delimiting strings. If zero, double quote (") is used
  bColSeparator   AS BYTE           ' character separating fields. If zero, comma (,) is used
  #ENDIF
  teol            AS EOLBYTES       ' bytes defining the end of a row. The default is CRLF.
END TYPE

' This structure is used by the GCM_SAVEFORMAT messages
TYPE GRIDSAVEFORMAT   ' gsf
  #IF %DEF(%UNICODE)
  szFile            AS WSTRINGZ * %MAX_PATH ' character delimiting strings. If zero, double quote (") is used
  #ELSE
  szFile            AS ASCIIZ * %MAX_PATH   ' character delimiting strings. If zero, double quote (") is used
  #ENDIF
  fEncodeImages     AS LONG
  fSaveColText      AS LONG
  fSaveRowText      AS LONG
  fSaveCellText     AS LONG
  xPos              AS LONG
  yPos              AS LONG
END TYPE

'------------------ [ Custom Styles/Messages/Notifications ] -------------------

' Phnx_Grid_Class32 styles
' ------------------------
%GCS_CAPTION                                = &H0001??
' The control has a caption.

%GCS_HORZGRIDLINES                          = &H0002??
' The control displays horizontal grid lines.

%GCS_VERTGRIDLINES                          = &H0004??
' The control displays vertical grid lines.

%GCS_FLATHEADER                             = &H0008??
' The headers of the control have a flat appearence.

%GCS_READONLY                               = &H0010??
' The contents of the grid control can be copied but not edited.

%GCS_TOOLTIPS                               = &H0020??
' The grid control creates a tooltip control when it is created.  The
' tooltip control always uses the LPSTR_TEXTCALLBACK flag even
' even when it is assigned via the GM_SETTOOLTIPS message.

%GCS_TWOCLICKACTIVATE                       = &H0040??
' Clicking an unselected item selects the item, and clicking the item
' a second time activates label editing. If this style is not set, clicking
' the item once, activates label editing.

%GCS_CYCLESTATEONCLICK                      = &H0080??
' The grid control changes the state image of an item each time
' the state image is clicked.

%GCS_DRAGDROPEDIT                           = &H0100??
' Enables drag and drop editing of grid items.

%GCS_CUSTOMEDITOR                           = &H0200??
' Enables customization of the grid control via built-in property editors.

%GCS_DISABLENOSCROLL                        = &H0400??
' The scroll bars are always displayed even when all items are visible.
' To display a scroll bar, set the WS_HSCROLL or WS_VSCROLL style.

%GCS_TEXTCALLBACK                           = &H0800??
' The control sends the GCN_GETDISPINFO notification to retrieve
' item information that is stored by the application rather than the control.

%GCS_SETSELECTION                           = &H1000??
' All text in the label-editing control is selected when label editing begins.

%GCS_SHOWSELALWAYS                          = &H2000??
' The selection, if any, is always shown, even if the control does not have the focus.

%GCS_SILENT                                 = &H4000??
' The control does not beep when an error occurs in a masked cell.  If this style is not set, the control beeps whenever an error
' occurs.

%GCS_VIRTUAL                                = &H8000??

' Phnx_Grid_Class32 extended styles
' ---------------------------------
%GCS_EX_LABELEDITONFOCUS                    = &H0001??
' In-place editing of a grid item's text is activated
' immediately when the item gains the focus.

%GCS_EX_CLIPCELL                            = &H0002??
' Text and images in a cell are automatically clipped when they exceed
' the boundaries of the cell.

%GCS_EX_AUTOSIZEONEXPAND                    = &H0004??
' Causes the column containing a node to automatically resize when the
' list of child rows expands or collapses.

%GCS_EX_BUSYTIP                             = &H0008??
' The grid control displays information when an item that requests
' a large number of child items on demand is expanded.

%GCS_EX_GRIDFILLED                          = &H0010??
' The grid control displays grid lines in the area of its client area
' that is not occupied by grid cells when GCS_HORZGRIDLINES or
' GCS_HORZGRIDLINES or both are set.

%GCS_EX_SINGLESELECT                        = &H0020??
' The grid control allows only one item at a time to be selected.

%GCS_EX_NOFOCUSRECT                         = &H0040??
' The grid control does not draw a focus rectangle around the cell the focus.

%GCS_EX_NONODELINES                         = &H0080??
' The grid control does not draw lines between parent and child cells to show
' the hierarchy of items.

%GCS_EX_LABELEDITFIXED                      = &H0100??
' The width of the label-edit control remains fixed as text is typed even when
' the text extends beyond the borders of a cell.  The text in the label-edit
' control scrolls automatically when it is wider than the cell.

%GCS_EX_LABELEDITSORT                       = &H0200??
' Sorts the strings in the label-edit drop-down list alphabetically.

%GCS_EX_LABELEDITENDONCLICK                 = &H0400??
' The grid control ends label-editing when an item from the drop-down list is clicked.

%GCS_EX_BKGNDCOLORTOBORDER                  = &H0800??
' The background color of a cell includes the borders of the cell
' when the color is not the default.  As a result, adjacent cell with
' the same background color appear as one cell.

' Phnx_Grid_Class32 messages
' --------------------------
%GCM_INITDONE                               = %WM_USER + &H0064??
' Purpose: Posted to ensure that the grid control draws its entire client
'          area to the memory device contexts it manages when the
'          control is initially created.  The control may not paint its entire
'          client area during the initial WM_PAINT message because it
'          ma
' wParam:  Reason why message was posted
' lParam:  N/A
' Return:  No return value

%GCM_GETITEM                                = %WM_USER + &H0065??
' Purpose: Retrieves some or all of a grid item's attributes.
' wParam:  N/A
' lParam:  Address of GRIDITEMEX structure
' Return:  TRUE if successful, FALSE on failure.

%GCM_SETITEM                                = %WM_USER + &H0066??
' Purpose: Sets some or all of a grid item's attributes.
' wParam:  If TRUE, sets the attributes of selected items
' lParam:  Address of GRIDITEMEX structure
' Return:  TRUE if successful, FALSE on failure.

%GCM_GETITEMSTATE                           = %WM_USER + &H0067??
' Purpose: Retrieves the state of an item in a grid control.
' wParam:  N/A
' lParam:  Address of a GRIDITEMEX structure.
' Return:  The current state of the item.

%GCM_SETITEMSTATE                           = %WM_USER + &H0068??
' Purpose: Changes the state of an item in a grid control.
' wParam:  N/A
' lParam:  Address of a GRIDITEMEX structure.
' Return:  TRUE if successful or FALSE otherwise.

%GCM_GETLABELEDITCONTROL                    = %WM_USER + &H0069??
' Purpose: Sent to retrieve the handle to the control being used
'          to edit a grid item's text.
' wParam:  N/A
' lParam:  N/A
' Return:  Handle to the label editing control if successful, or NULL otherwise.

%GCM_GETDROPLISTCONTROL                     = %WM_USER + &H006A??
' Purpose: Sent to retrieve the handle of the drop-list control that is being used
'          to edit a grid item's text. The value returned is the handle of the child
'          listbox control of the drop-list container.
' wParam:  N/A
' lParam:  N/A
' Return:  Handle to the child listbox control if successful, or NULL otherwise.

%GCM_GETIMAGELIST                           = %WM_USER + &H006B??
' Purpose: Retrieves the image list associated with a grid control.
' wParam:  Type of image list
' lParam:  N/A
' Return:  Handle to the image list associated with the grid control or NULL.

%GCM_SETIMAGELIST                           = %WM_USER + &H006C??
' Purpose: Assigns an image list to a grid control.
' wParam:  Type of image list
' lParam:  Handle to the image list to assign
' Return:  The handle to the image list previously associated with the control if successful, or NULL otherwise.

%GCM_GETITEMRECT                            = %WM_USER + &H006D??
' Purpose: Retrieves the bounding rectangle of a grid  item in the current view.
'          When the message is sent, the nLeft and nTop members of the RECT
'          structure specify the indices of the item.
'          The return value is nonzero if the item is visible with the high-order word is zero
'          if the item is fully visible, and one if the item is partially visible.
' wParam:  lowrd specifies the grid item type
' lParam:  address of RECT structure to receive coordinates.
' Return:  Returns TRUE if successful, or FALSE otherwise.

%GCM_INSERTCOLUMN                           = %WM_USER + &H006E??
' Purpose: Sent to insert columns in a grid control.
'          The new column is inserted before the specified column.
'          Set iCol to -1 to insert the column at the end of the grid.
' wParam:  Position at which to insert the new column
' lParam:  Number of columns to insert
' Return:  Returns TRUE if successful, or FALSE otherwise.

%GCM_INSERTROW                              = %WM_USER + &H006F??
' Purpose: Sent to insert rows in a grid control.
'          The new row is inserted before the specified row.
'          Set iRow to -1 to insert the row at the end of the grid.
' wParam:  Position at which to insert the new row
' lParam:  Number of columns to insert
' Return:  Returns TRUE if successful, or FALSE otherwise.

%GCM_DELETECOLUMN                           = %WM_USER + &H0070??
' Purpose: Sent to a grid control to delete one column or
'          a range of columns.
' wParam:  Zero-based index of first column in range
' lParam:  Zero-based index of last column in range.
' Return:  No return value.

%GCM_DELETEROW                              = %WM_USER + &H0071??
' Purpose: Sent to a grid control to delete one row or
'          a range or rows.
' wParam:  Zero-based index of first row in range
' lParam:  Zero-based index of last row in range.
' Return:  No return value.

%GCM_DELETEALLITEMS                         = %WM_USER + &H0072??
' Purpose: Deletes all columns and rows from a grid control.
' wParam:  N/A
' lParam:  N/A
' Return:  No return value.

%GCM_REFRESHGRID                            = %WM_USER + &H0073??
' Purpose: Sent to redraw the visible area of the grid.  This message
'          recalculates the position and dimension of the various grid
'          components before redrawing them.
' wParam:  N/A
' lParam:  N/A
' Return:  No return value.

%GCM_CLEARCONTENTS                          = %WM_USER + &H0074??
' Purpose: Sent to clear the contents of the specified items.
' wParam:  lowrd = type of item. Set to &HFFFF to clear all items. hiwrd = set format to default if TRUE
' lParam:  address of GRIDRANGE structure specifying the range.
' Return:  N/A

%GCM_LOADTEXTFILE                           = %WM_USER + &H0075??
' Purpose: Loads data from a Comma Separated Values (CSV) file or
'          a Tab Delimited file.  The lobyte of wParam can be used
'          to specify the delimiter.
' wParam:  lobyte = optional field separator (set to zero to autodetect). hiwrd = headers flag (set to TRUE if first line in file defines t
'          he column headers)
' lParam:  Fully qualified path and filename
' Return:  FALSE if file was opened successfully, or nonzero on failure.

%GCM_SAVETEXTFILE                           = %WM_USER + &H0076??
' Purpose: Saves data in grid as Comma Separated Values (CSV) or
'          as Tab Delimited values.  The wParam parameter can be
'          to specify the address of a GRIDGETTEXT structure
'          to set the delimiter and range of items to be saved.
' wParam:  address of optional GRIDGETTEXT structure or null to save the entire grid
' lParam:  Fully qualified path and filename
' Return:  FALSE if data was saved successfully, of nonzero on failure.

%GCM_GETITEMTEXT                            = %WM_USER + &H0077??
' Purpose: Sent to retrieve data from the grid control.
'          Set lParam to zero to retrieve the size of the
'          buffer required for the text (including the
'          terminating null).
' wParam:  Address of optional GRIDGETTEXT structure.
' lParam:  Address of buffer to receive data
' Return:  Size of buffer needed or count of bytes retrieved.

%GCM_SETITEMTEXT                            = %WM_USER + &H0078??
' Purpose: Sent to fill specific portions of the grid control with data.
'          The data must be null-terminated.
' wParam:  Address of GRIDSETTEXT structure.  Set to null to use the defaults
' lParam:  Address of buffer with null-terminated data.
' Return:  FALSE if successful, TRUE on failure.

%GCM_MERGECELLS                             = %WM_USER + &H0079??
' Purpose: Sent to merge or unmerge cells. If wParam is FALSE, the cells are
'          merged.  If wParam is TRUE, the cells are unmerged.
' wParam:  If TRUE, the cells are unmerged
' lParam:  address of RECT structure specifying the range.
' Return:  Returns TRUE if successful, or FALSE otherwise.

%GCM_REDRAWITEMS                            = %WM_USER + &H007A??
' Purpose: Sent to redraw items.
' wParam:  lowrd specifies the grid item to redraw or zero to redraw all items
' lParam:  address of RECT structure specifying the range or Null
' Return:  No return value.

%GCM_SETTOOLTIPS                            = %WM_USER + &H007B??
' Purpose: Sent to associate a tooltip control with the grid condrol.
' wParam:  Handle to the tooltip control
' lParam:  N/A
' Return:  No return value.

%GCM_GETTOOLTIPS                            = %WM_USER + &H007C??
' Purpose: Sent to retrieve the handle to the tooltip control, if any, associated
'          with the grid control.
' wParam:  N/A
' lParam:  N/A
' Return:  Returns the handle to the tooltip control or NULL if the grid control has no associated tooltip.

%GCM_GETVISIBLERANGE                        = %WM_USER + &H007D??
' Purpose: Sent to retrieve the indices of the leftmost, topmost, rightmost,
'          and bottommost visible items that are not pinned.  The rightmost
'          and bottommost items may be partially visible.
' wParam:  N/A
' lParam:  address of RECT structure to receive item indices
' Return:  No return value.

%GCM_SETFIRSTVISIBLECELL                    = %WM_USER + &H007E??
' Purpose: Sent to set the cell that is located at the leftmost column
'          and topmost row of a grid control.
' wParam:  zero-based column index of cell
' lParam:  zero-based row index of cell
' Return:  TRUE if successful, FALSE otherwise.

%GCM_ISCELLMERGED                           = %WM_USER + &H007F??
' Purpose: Sent to determine whether or not the specified cell is merged.
'          On return, the RECT structure contains the vectors of the cell
'          if it is merged.
' wParam:  N/A
' lParam:  address of RECT structure specifying column and row index of cell
' Return:  TRUE if cell is merged, FALSE if it is not.

%GCM_SETFOCUSCELL                           = %WM_USER + &H0080??
' Purpose: Sent to set the focus to the specified cell.  The cell
'          is also selected if it is not already selected.  The focus
'          is set to a different cell if the specified cell is disabled,
'          locked, or its dimensions make it invisible.
' wParam:  zero-based column index of cell
' lParam:  zero-based row index of cell
' Return:  TRUE if the focus was set to a cell that is different from the specified cell.

%GCM_GETFOCUSCELL                           = %WM_USER + &H0081??
' Purpose: Sent to retrieve the cell with the focus.
' wParam:  address of variable to receive the column index
' lParam:  address of variable to receive the row index
' Return:  TRUE if successful, FALSE otherwise.

%GCM_ENSUREVISIBLE                          = %WM_USER + &H0082??
' Purpose: Sent to ensure that a grid item is either entirely or partially
'          visible, scrolling the grid control if necessary.  Only the
'          iLeft, iTop, dwHitCode members of the GRIDITEMEX
'          are valid.
' wParam:  If TRUE, no scrolling occurs if the item is partially visible
' lParam:  address of GRIDITEMEX structure
' Return:  No return value.

%GCM_GETCOLUMNCOUNT                         = %WM_USER + &H0083??
' Purpose: Sent to retrieve the number of columns in a grid control.
'          The return value does not include the marker column.
' wParam:  N/A
' lParam:  N/A
' Return:  The number of columns.

%GCM_GETROWCOUNT                            = %WM_USER + &H0084??
' Purpose: Sent to retrieve the number of rows in a grid control.
'          The return value does not include the marker row.
' wParam:  N/A
' lParam:  N/A
' Return:  The number of rows.

%GCM_GETTEXTCOLOR                           = %WM_USER + &H0085??
' Purpose: Sent to retrieve the default text color of cells and headers
'          in a grid control.
' wParam:  N/A
' lParam:  N/A
' Return:  The current text color.

%GCM_SETTEXTCOLOR                           = %WM_USER + &H0086??
' Purpose: Sent to set the default text color of cells and headers
'          in a grid control.
' wParam:  if TRUE, sets the header text color
' lParam:  new text color or &HFFFFFFFF???
' Return:  The old text color.

%GCM_GETBACKCOLOR                           = %WM_USER + &H0087??
' Purpose: Sent to retrieve the default back color of cells and headers
'          in a grid control.
' wParam:  N/A
' lParam:  N/A
' Return:  The current back color.

%GCM_SETBACKCOLOR                           = %WM_USER + &H0088??
' Purpose: Sent to set the default back color of cells and headers
'          in a grid control.
' wParam:  if TRUE, sets the header back color
' lParam:  new back color or &HFFFFFFFF???
' Return:  The old back color.

%GCM_GETGRIDCOLOR                           = %WM_USER + &H0089??
' Purpose: Sent to retrieve the color of the grid lines in a grid control.
' wParam:  N/A
' lParam:  N/A
' Return:  The current grid color.

%GCM_SETGRIDCOLOR                           = %WM_USER + &H008A??
' Purpose: Sent to set the color of the grid lines in a grid control.
' wParam:  N/A
' lParam:  new grid color or &HFFFFFFFF???
' Return:  The old grid color.

%GCM_GETURLCOLOR                            = %WM_USER + &H008B??
' Purpose: Sent to retrieve the default text color of Universal Resource Locators
'          (URL) in a grid control.
' wParam:  N/A
' lParam:  N/A
' Return:  The current URL color.

%GCM_SETURLCOLOR                            = %WM_USER + &H008C??
' Purpose: Sent to set the default text color of Universal Resource Locators
'          (URL) in a grid control.
' wParam:  N/A
' lParam:  new URL color or &HFFFFFFFF???
' Return:  The old URL color.

%GCM_GETEDITRECTCOLOR                       = %WM_USER + &H008D??
' Purpose: Sent to retrieve the color of the rectangle that is drawn
'          around editable cells with the focus in a grid control.
' wParam:  N/A
' lParam:  N/A
' Return:  The current edit rectangle color.

%GCM_SETEDITRECTCOLOR                       = %WM_USER + &H008E??
' Purpose: Sent to set the color of the rectangle that is drawn
'          around editable cells with the focus in a grid control.
' wParam:  N/A
' lParam:  new edit rectangle color or &HFFFFFFFF???
' Return:  The old edit rectangle color.

%GCM_GETFIELDMETRIC                         = %WM_USER + &H008F??
' Purpose: Sent to retrieve the ascii characters delimiting strings, columns,
'          and rows in a grid control.  The double quote (") is the default
'          string delimiter, the comma (,) is the default column separator
'          and the CRLF pair is the default row separator.
' wParam:  N/A
' lParam:  address of GRIDFIELDMETRIC structure
' Return:  Returns TRUE if successful, or FALSE otherwise.

%GCM_SETFIELDMETRIC                         = %WM_USER + &H0090??
' Purpose: Sent to set the ascii characters delimiting strings, columns, and
'          rows in a grid control.  The double quote (") is the default string
'          delimiter, the comma (,) is the default column separator and the
'          CRLF pair is the default row separator.
' wParam:  N/A
' lParam:  address of GRIDFIELDMETRIC structure
' Return:  Always returns TRUE.

%GCM_EDITLABEL                              = %WM_USER + &H0091??
' Purpose: Sent to begin in-place editing of the specified grid item's text.
'          The message implicitly selects and focuses the specified item.
'          This message fails if the grid control does not have the input
'          focus.  Send the WM_CANCELMODE message to cancel editing.
' wParam:  N/A
' lParam:  address of GRIDITEMEX structure
' Return:  Returns TRUE if successful, or FALSE otherwise.

%GCM_SCROLL                                 = %WM_USER + &H0092??
' Purpose: Sent to scroll the contents of a grid control.
' wParam:  number of columns to scroll
' lParam:  number of rows to scroll
' Return:  No return value.

%GCM_SELECTITEM                             = %WM_USER + &H0093??
' Purpose: Sent to select items in a grid control.  If wParam is TRUE,
'          this message does not deselect previously selected items
'          or change the focus.
' wParam:  if TRUE, previously selected items are not deselected
' lParam:  address of GRIDITEMEX structure
' Return:  No return value.

%GCM_GETHEADERIDEALSIZE                     = %WM_USER + &H0094??
' Purpose: Retrieves the ideal size of a grid item.  Valid item types are
'          GHT_CAPTION, GHT_COLHEADER and GHT_ROWHEADER.
'          The width or height is returned if the value of the  pwWidth
'          or pwHeight member is set to the address of a variable.
' wParam:  N/A
' lParam:  address of GRIDHEADERSIZE structure
' Return:  Returns TRUE if successful, or FALSE otherwise.

%GCM_SETHEADERIDEALSIZE                     = %WM_USER + &H0095??
' Purpose: Sent to set the ideal size of a grid item.  Valid item types are
'          GHT_CAPTION, GHT_COLHEADER and GHT_ROWHEADER.
'          The width or height is set if the value of the  pwWidth or pwHeight
'          member is set to TRUE.
' wParam:  N/A
' lParam:  address of GRIDHEADERSIZE structure
' Return:  Returns TRUE if successful, or FALSE otherwise.

%GCM_GETHEADERHEIGHT                        = %WM_USER + &H0096??
' Purpose: Retrieves the height (GHT_COLHEADER, GHT_CAPTION)
'          or width (GHT_ROWHEADER) of a grid item.
' wParam:  lowrd specifies the type of item
' lParam:  N/A
' Return:  The height/width of the grid item.

%GCM_SETHEADERHEIGHT                        = %WM_USER + &H0097??
' Purpose: Sets the height (GHT_COLHEADER, GHT_CAPTION)
'          or width (GHT_ROWHEADER) of a grid item.
' wParam:  lowrd specifies the type of item
' lParam:  new height/width of item
' Return:  TRUE if successful, FALSE otherwise.

%GCM_GETDEFAULTSIZE                         = %WM_USER + &H0098??
' Purpose: Retrieves the default size of the specified item.  Valid item types are
'          GHT_CAPTION, GHT_COLHEADER and GHT_ROWHEADER.
' wParam:  lowrd specifies the type of item
' lParam:  False=user defaults. True=grid defaults
' Return:  The default width (lowrd) and height (hiwrd) of the item if successful.

%GCM_GETSELECTION                           = %WM_USER + &H0099??
' Purpose: Sent to retrieve the indices of selected items in a grid control.
'          Set lParam to NULL to retrieve the number of selected items
'          of the specified type.
' wParam:  lowrd specifies the type of item
' lParam:  address of an array of POINTAPI structs to receive item indices
' Return:  The number of selected items if successful.

%GCM_HITTEST                                = %WM_USER + &H009A??
' Purpose: Sent to determine which item of the grid control, if any, is at a
'          specified position.
' wParam:  N/A
' lParam:  Address of GRIDHITTEST structure
' Return:  The hit-test value.

%GCM_ISRECTANGULARSEL                       = %WM_USER + &H009B??
' Purpose: Sent to determine whether or not the selection is contiguous (rectangular).
' wParam:  N/A
' lParam:  Address of optional RECT structure to receive indices.
' Return:  TRUE if selection is contiguous, FALSE if it is not.

%GCM_SORT                                   = %WM_USER + &H009C??
' Purpose: Sent to sort the items in a grid control.
' wParam:  Index of column to sort
' lParam:  N/A
' Return:  No return value.

%GCM_SETHEADERDISPLAYBASE                   = %WM_USER + &H009D??
' Purpose: Sets the base number for the default text that is displayed in column
'          and row headers.  The default base is zero.
' wParam:  N/A
' lParam:  The new default base
' Return:  The return value is the old default base,

%GCM_GETHEADERDISPLAYBASE                   = %WM_USER + &H009E??
' Purpose: Retrieves the base number for the default text that is displayed in column
'          and row headers.
' wParam:  N/A
' lParam:  N/A
' Return:  The return value is the current default base.

%GCM_GETITEMTEXTEX                          = %WM_USER + &H009F??
' Purpose: Sent to retrieve data from the grid control.
'          Set lParam to zero to retrieve the size of the
'          buffer required for the text (including the
'          terminating null).
' wParam:  Address of optional GRIDGETTEXTEX structure.
' lParam:  Address of buffer to receive data
' Return:  Size of buffer needed or count of bytes retrieved.

%GCM_SETITEMTEXTEX                          = %WM_USER + &H00A0??
' Purpose: Sent to fill specific portions of the grid control with data.
' wParam:  Address of GRIDSETTEXTEX structure.
' lParam:  Not used; must be zero
' Return:  FALSE if successful, TRUE on failure.

%GCM_DISPLAYFORMATEDITOR                    = %WM_USER + &H00A1??
' Purpose: Sent to display the Format Editor.  This message displays the
'          editor  even when the GCS_CUSTOMEDITOR style is not set.
' wParam:  N/A
' lParam:  N/A
' Return:  No return value.

%GCM_LOADTEXTFILEEX                         = %WM_USER + &H00A2??
' Purpose: Loads data from the specified file.  By  default, the file is assumed
'          to consist of comma separated values (CSV) or a tab delimited values.
'          with strings delimited by the double quote character.
' wParam:  Address of GRIDSETTEXTEX structure.
' lParam:  Not used; must be zero
' Return:  FALSE if file was opened successfully, or nonzero on failure.

%GCM_SAVETEXTFILEEX                         = %WM_USER + &H00A3??
' Purpose: Saves data in grid to the specified file.  By default, the data is saved
'          using he fiield metrics set by the last GCM_SETFIELDMETRIC message.
'
' wParam:  Address of optional GRIDGETTEXTEX structure.
' lParam:  Fully qualified path and filename
' Return:  FALSE if data was saved successfully, of nonzero on failure.

%GCM_SETLABELEDITDELAYTIME                  = %WM_USER + &H00A4??
' Purpose: Sets the  label-editing delay time.  The label-editing delay time
'          is the number of milliseconds that may occur between a mouse
'          click and the start of label-editing.
'          The default is the GetDoubleClickTime value.
' wParam:  The new label-editing delay time in milliseconds
' lParam:  If True, the value is set to the default
' Return:  The return value is the old label-editing dalay time in milliseconds.

%GCM_GETLABELEDITDELAYTIME                  = %WM_USER + &H00A5??
' Purpose: Retrieves the current label-editing delay time.  The label-editing delay time
'          is the number of milliseconds that may occur between a mouse click and
'          the start of label-editing.
'          The default is the GetDoubleClickTime value.
' wParam:  Not used; must be zero
' lParam:  Not used; must be zero
' Return:  The return value is the current label-editing delay time in milliseconds.

%GCM_GETMASKBACKCOLOR                       = %WM_USER + &H00A6??
' Purpose: Gets the current background color for the masked label-editing control.
'          If wParam is TRUE, the message gets the background color of literals.
'          If wParam is FALSE, the message gets the background color of non-literals.
'
' wParam:  A flag that specifies the type of text the color should be applied to
' lParam:  Not used; must be zero
' Return:  The return value is the current background color.

%GCM_SETMASKBACKCOLOR                       = %WM_USER + &H00A7??
' Purpose: Sets the current background color for the masked label-editing control.
'          If wParam is TRUE, the message sets the background color of literals.
'          If wParam is FALSE, the message sets the background color of non-literals.
'
' wParam:  A flag that specifies the type of text the color should be applied to
' lParam:  New background color
' Return:  The return value is the old background color.

%GCM_GETMASKTEXTCOLOR                       = %WM_USER + &H00A8??
' Purpose: Gets the current text color for the masked label-editing control.
'          If wParam is TRUE, the message gets the text color of literals.
'          If wParam is FALSE, the message gets the text color of non-literals.
'
' wParam:  A flag that specifies the type of text the color should be applied to
' lParam:  Not used; must be zero
' Return:  The return value is the current text color

%GCM_SETMASKTEXTCOLOR                       = %WM_USER + &H00A9??
' Purpose: Sets the current text color for the masked label-editing control.
'          If wParam is TRUE, the message sets the text color of literals.
'          If wParam is FALSE, the message sets the text color of non-literals.
'
' wParam:  A flag that specifies the type of text the color should be applied to
' lParam:  New text color
' Return:  The return value is the old text color.

%GCM_SETEXTENDEDGRIDSTYLE                   = %WM_USER + &H00AA??
' Purpose: Sets extended styles in a grid control.
' wParam:  Specifies which styles in lParam are to be affected
' lParam:  Specifies the extended grid control styles to set
' Return:  The return value is the previous extended grid control styles.

%GCM_GETEXTENDEDGRIDSTYLE                   = %WM_USER + &H00AB??
' Purpose: Gets the extended styles that are currently in use for a given grid control.
' wParam:  Not used; must be zero
' lParam:  Not used; must be zero
' Return:  The return value is the styles currently in use for a given grid control.

%GCM_SHOWLABELEDITDROPLIST                  = %WM_USER + &H00AC??
' Purpose: Sent to show or hide the label-editing drop-list window.
' wParam:  If True, the drop-list window is hidden
' lParam:  Not used; must be zero
' Return:  Returns TRUE if successful, or FALSE otherwise.

%GCM_ISITEMVISIBLE                          = %WM_USER + &H00AD??
' Purpose: Sent to determine whether a grid item is visible within the client area of the grid.
'          The return value is nonzero if the item is visible with the high-order word is zero
'          if the item is fully visible, and one if the item is partially visible.
' wParam:  Not used; must be zero
' lParam:  Address of a GRIDCELL structure
' Return:  Returns TRUE if item is visible; FALSE if it is not.

%GCM_GETNODEBUTTONCOLOR                     = %WM_USER + &H00AE??
' Purpose: Sent to retrieve the current color of node buttons in a grid control.
' wParam:  Not used, must be zero
' lParam:  Not used, must be zero
' Return:  The return value is the current node button color.

%GCM_SETNODEBUTTONCOLOR                     = %WM_USER + &H00AF??
' Purpose: Sent to set the current color of node buttons in a grid control.
' wParam:  Not used, must be zero
' lParam:  New node button color or &HFFFFFFFF???
' Return:  The return value is the old node button color.

%GCM_GETNODELINECOLOR                       = %WM_USER + &H00B0??
' Purpose: Sent to retrieve the current color of node lines in a grid control.
' wParam:  Not used, must be zero
' lParam:  Not used, must be zero
' Return:  The return value is the current node line color.

%GCM_SETNODELINECOLOR                       = %WM_USER + &H00B1??
' Purpose: Sent to set the current color of node lines in a grid control.
' wParam:  Not used, must be zero
' lParam:  New node line color or &HFFFFFFFF???
' Return:  The return value is the old node line color.

%GCM_EXPAND                                 = %WM_USER + &H00B2??
' Purpose: Sent to expand or collapse the list of child rows associated with the specified parent cell, if any.
' wParam:  Not used, must be zero
' lParam:  Address of GRIDEXPAND structure
' Return:  The return value is nonzero if the operation was successful, or zero otherwise.

%GCM_SETITEMPARENT                          = %WM_USER + &H00B3??
' Purpose: Sent to set the parent cell of a range of cells.
' wParam:  Not used, must be zero
' lParam:  Address of GRIDITEMPARENT structure
' Return:  The return value is nonzero if the operation was successful, or zero otherwise.

%GCM_GETNEXTITEM                            = %WM_USER + &H00B4??
' Purpose: Retrieves the cell that bears the specified relationship to the reference cell.
'          On input, the GRIDCELL structure is set to the reference cell.  On return, the
'          GRIDCELL structure is changed the cell with the specified relationship.
' wParam:  Flag specifying the item to retrieve
' lParam:  Address of a GRIDCELL structure
' Return:  The return value is True if successful, or False otherwise.

%GCM_GETSELECTBACKCOLOR                     = %WM_USER + &H00B5??
' Purpose: Sent to retrieve the background color of selected cells in a grid control.
' wParam:  N/A
' lParam:  N/A
' Return:  The current background color of selected cells.

%GCM_SETSELECTBACKCOLOR                     = %WM_USER + &H00B6??
' Purpose: Sent to set the background color of selected cells in a grid control.
' wParam:  N/A
' lParam:  new background color or &HFFFFFFFF???
' Return:  The old background color of selected cells.

%GCM_SAVEFORMAT                             = %WM_USER + &H00B7??
' Purpose: Sent to save the contente of the grid as a format file.
' wParam:  Not used, must be zero
' lParam:  Address of a GRIDSAVEFORMAT structure
' Return:  The return value is nonzero if an error occurred.

%GCM_LOADFORMATFROMFILE                     = %WM_USER + &H00B8??
' Purpose: Initializes a grid control from the specifed file.
' wParam:  Not used, must be zero
' lParam:  Pointer to the fully qualified path and file name
' Return:  The return value is zero on success; nonzero on failure.

%GCM_SETDEFAULTSIZE                         = %WM_USER + &H00B9??
' Purpose: Sets the default size of the specified item.  Valid item types are
'          GHT_CAPTION, GHT_COLHEADER and GHT_ROWHEADER.  Set the new value to
'          &HFFFF?? to have the grid calculate the default.
' wParam:  lowrd specifies the type of item
' lParam:  The default width (lowrd) and height (hiwrd) of the item
' Return:  No return value.

%GCM_GETNODEBACKCOLOR                       = %WM_USER + &H00BA??
' Purpose: Sent to retrieve the current background color of node buttons in a grid control.
' wParam:  Not used, must be zero
' lParam:  Not used, must be zero
' Return:  The return value is the current node button background color.

%GCM_SETNODEBACKCOLOR                       = %WM_USER + &H00BB??
' Purpose: Sent to set the current background color of node buttons in a grid control.
' wParam:  Not used, must be zero
' lParam:  New node button background color or &HFFFFFFFF???
' Return:  The return value is the old node button background color.

%GCM_GETIMAGECOUNT                          = %WM_USER + &H00BC??
' Purpose: Sent to retrieve the number of images in the image collection.
' wParam:  Not used, must be zero
' lParam:  Not used, must be zero
' Return:  The return value is the number of images in the collection.

%GCM_GETIMAGES                              = %WM_USER + &H00BD??
' Purpose: Sent to retrieve all the images in the collection.  The buffer that
'          is passed in lParam must be large enough to hold all the imaeges.
'          Embedded pointers that are returned must not be freed.
' wParam:  Not used, must be zero
' lParam:  Pointer to an array of GRIDIMAGEITEM structures
' Return:  The return value is True on success; False on failure.

%GCM_ADDIMAGE                               = %WM_USER + &H00BE??
' Purpose: Sent to add one or more images to the image collection.  On return,
'          the rImageKey member of the GRIDIMAGEITEM structure contains the
'          identifier of the image in the collection.
' wParam:  The number of images to add.
' lParam:  Address of an array of GRIDIMAGEITEM structures.
' Return:  The return value is True on success; False on failure.

%GCM_DELETEIMAGE                            = %WM_USER + &H00BF??
' Purpose: Sent to delete an image from the image collection.  If the identifier
'          passed in wParam is IID_NULL, all images are deleted.
' wParam:  Pointer to the identifier of the image in the collection
' lParam:  Not used, must be zero
' Return:  The return value is True on success; False on failure.

%GCM_GETNEXTIMAGE                           = %WM_USER + &H00C0??
' Purpose: Sent to enumerate the images in the collection.  To return  the
'          first image in the collection, set the identifier pass in wParam
'          to IID_NULL.
'          Embedded pointers that are returned must not be freed.
' wParam:  Pointer to the identifier of the image in the collection
' lParam:  Address of a GRIDIMAGEITEM structure.
' Return:  The return value is True on success; False on failure.

%GCM_GETPINNEDCELLBACKCOLOR                 = %WM_USER + &H00C1??
' Purpose: Sent to retrieve the current background color of pinned cells in a grid control.
' wParam:  Not used, must be zero
' lParam:  Not used, must be zero
' Return:  The return value is the current background color of pinned cells.

%GCM_SETPINNEDCELLBACKCOLOR                 = %WM_USER + &H00C2??
' Purpose: Sent to set the current background color of pinned cells in a grid control.
' wParam:  Not used, must be zero
' lParam:  New background color of pinned cells or &HFFFFFFFF???
' Return:  The return value is the old background color of pinned cells.

%GCM_GETFORMATFILE                          = %WM_USER + &H00C3??
' Purpose: Sent to retrieve the name of the file that contains images as Base64 encoded data.
'          The buffer that is passed via lParam must be at least MAX_PATH in size.
' wParam:  Not used, must be zero
' lParam:  Address of variable that receives the name of the file
' Return:  No return value.

%GCM_SETFORMATFILE                          = %WM_USER + &H00C4??
' Purpose: Sent to set the name of the format file that contains images as Base64 encoded data.
' wParam:  Not used, must be zero
' lParam:  Address of buffer with name of format file
' Return:  No return value.

%GCM_CLONEIMAGES                            = %WM_USER + &H00C5??
' Purpose: Returns a copy of the image collection of a grid control.
'          The collection that be freed using the GCM_FREEIMAGES
'          message.
' wParam:  Address of variable for the number of images or null
' lParam:  Not used, must be zero
' Return:  The return value is a pointer to a copy of the image collection(an array of GRIDIMAGEITEM structures).

%GCM_FREEIMAGES                             = %WM_USER + &H00C6??
' Purpose: Frees an image collection.  If lParam is null, the current collection
'          in the grid control is freed.
' wParam:  Number of images in collection
' lParam:  Ponter to image collection to free or null
' Return:  No return value.

%GCM_SETIMAGES                              = %WM_USER + &H00C7??
' Purpose: Sets the image collection in a grid control.  The previous collection is freed.
' wParam:  The number of images in the collection
' lParam:  Pointer to the image collection(array of GRIDIMAGEITEM structures)
' Return:  No return value.

%GCM_LOADFORMATFROMMEMORY                   = %WM_USER + &H00C8??
' Purpose: Initializes a grid control from the specified buffer.
' wParam:  The size of buffer
' lParam:  Pointer to the buffer used to initialize the grid
' Return:  The return value is zero on success; nonzero on failure.

%GCM_GETIMAGELISTEX                         = %WM_USER + &H00C9??
' Purpose: Retrieves the ImageList associated with a grid control.
'          The embedded handle and pointer that are  returned must not be freed.
' wParam:  Type of image list
' lParam:  Pointer to a GRIDIMAGELIST structure
' Return:  No return value.

%GCM_SETIMAGELISTEX                         = %WM_USER + &H00CA??
' Purpose: Sets the specified ImageList associated with a grid control.  The previous ImageList is freed.
' wParam:  Type of image list
' lParam:  Ponter to a GRIDIMAGELIST structure
' Return:  No return value.

%GCM_CLONEIMAGELISTEX                       = %WM_USER + &H00CB??
' Purpose: Returns a copy of the ImageList associated with a grid control.
'          The embedded handle and pointer that are returned can be
'          freed using the GCM_FREEIMAGELISTEX message.
' wParam:  Handle only flag or type of ImageList if lParam is null
' lParam:  Pointer to a GRIDIMAGELIST structure
' Return:  No return value.

%GCM_FREEIMAGELISTEX                        = %WM_USER + &H00CC??
' Purpose: Frees the specified ImageList.  If lParam is null, the current
'          ImageList associated with the grid control is freed.
' wParam:  Type of image list
' lParam:  Ponter to a GRIDIMAGELIST structure or null
' Return:  No return value.

%GCM_GETIMAGESIZE                           = %WM_USER + &H00CD??
' Purpose: Retrieves the type and dimensions of the specified image.
' wParam:  Pointer to a textual image id(guid) or qualified path and filename
' lParam:  Address of a GRIDIMAGESIZE structure
' Return:  The return value is True on success; False on failure.

%GCM_GETIMAGE                               = %WM_USER + &H00CE??
' Purpose: Sent to retrieve the specified image in a collection.  To return  the first
'          image in the collection, set the identifier pass in wParam to  IID_NULL.
'          Embedded pointers that are returned must not be freed.
' wParam:  Pointer to the identifier of the image in the collection
' lParam:  Address of a GRIDIMAGEITEM structure.
' Return:  The return value is True on success; False on failure.

%GCM_GETSCROLLBARFLAGS                      = %WM_USER + &H00CF??
' Purpose: Sent to retrieve the scrollbar flags.  If the low-order word of the return value is
'          True, the horizontal scrollbar property is set.  If the hiigh-order word is True, the
'          vertical scrollbar property is set.
' wParam:  Not used, must be zero
' lParam:  Not used, must be zero
' Return:  The low-order word is the horizontal scrollbar property, and the high-order word is the vertical scrollbar property.

%GCM_SETSCROLLBARFLAGS                      = %WM_USER + &H00D0??
' Purpose: Sent to set the scrollbar flags.
' wParam:  Horizontal scrollbar flag
' lParam:  Vertical scrollbar flag
' Return:  No return value.

%GCM_HIDEHEADER                             = %WM_USER + &H00D1??
' Purpose: Sent to show or hide the specified header.  This message  hides an item by setting
'          its width or height to one pizel or less.  Valid values for the wParam parameter are
'          GHT_CAPTION, GHT_COLHEADER, and GHT_ROWHEADER.
' wParam:  Loword = the type of item; hiword = show flag
' lParam:  Address of a RECT structure with the range of items or null
' Return:  No return value.

%GCM_GETSCROLLABLEFLAGS                     = %WM_USER + &H00D2??
' Purpose: Sent to retrieve the scrollable flags.  If the low-order word of the return value is True,
'          the contents of the grid exceed the width of the client area.  If the hiigh-order word is
'          True, the contents exceed the height of the client area.
' wParam:  Not used, must be zero
' lParam:  Not used, must be zero
' Return:  The low-order word is the horizontal scrollable property, and the high-order word is the vertical scrollable property.

%GCM_PASTESPECIAL                           = %WM_USER + &H00D3??
' Purpose: Sent to copy the current content of the clipboard to the grid control.
' wParam:  Not used, must be zero
' lParam:  Ponter to a GRIDPASTE structure or null
' Return:  No return value.

%GCM_GETSELECTEDBOX                         = %WM_USER + &H00D4??
' Purpose: Retrieves the bounding box of the selected cells.  On return,
'          the GRIDRANGE structure contains the column and row indices of
'          the cells that form the tightest box that encloses the entire
'          selection.
' wParam:  Not used, must be zero
' lParam:  Address of a GRIDRANGE structure
' Return:  The return value is true on success; false on failure.

%GCM_ISANYSELECTED                          = %WM_USER + &H00D5??
' Purpose: Sent to determine whether at least one item of the specified
'          type is selected.
' wParam:  lowrd specifies the type of item
' lParam:  Not used, must be zero
' Return:  The return value is true if at least one item is selected.

' Phnx_Grid_Class32 notifications
' -------------------------------
%GCN_BEGINLABELEDIT                         = %WM_USER + &H0064??
' Purpose: Sent to the grid control's parent window about the start of label editing for an item. To replace
'          the built-in editor, set the value of the hWndEdit member of the GRIDLABELEDIT structure to
'          a valid control handle.
' wParam:  Identifier of control
' lParam:  Address of GRIDLABELEDIT structure
' Return:  To allow the user to edit the label, return FALSE.
'          To prevent the user from editing the label, return TRUE.

%GCN_ENDLABELEDIT                           = %WM_USER + &H0065??
' Purpose: Sent to a grid control's parent window when label editing for an item ends.
' wParam:  N/A
' lParam:  Address of GRIDLABELEDIT structure
' Return:  TRUE to set the item's label to the edited text.
'          FALSE to reject the edited text and revert to the original label.

%GCN_BEGINTRACK                             = %WM_USER + &H0066??
' Purpose: Sent by a grid control when the user begins to drag a divider in the control.
' wParam:  Identifier of control
' lParam:  Address of GRIDITEMACTIVATE structure
' Return:  Return TRUE to prevent the resizing of the header.

%GCN_ENDTRACK                               = %WM_USER + &H0067??
' Purpose: Notifies the parent window of a grid control that the user
'          has finished dragging a divider.
' wParam:  Identifier of control
' lParam:  Address of GRIDITEMACTIVATE structure
' Return:  N/A

%GCN_TRACK                                  = %WM_USER + &H0068??
' Purpose: Notifies the parent window of a grid control that the user
'          is dragging a divider.
' wParam:  Identifier of control
' lParam:  Address of GRIDITEMACTIVATE structure
' Return:  Return TRUE to cancel the resizing of the header.

%GCN_ITEMCLICK                              = %WM_USER + &H0069??
' Purpose: Sent to a grid control's parent window when the user clicks a grid item.
' wParam:  Identifier of control
' lParam:  Address of GRIDITEMACTIVATE structure
' Return:  Return TRUE to prevent the default operation.

%GCN_ITEMDOUBLECLICK                        = %WM_USER + &H006A??
' Purpose: Sent to a grid control's parent window when the user double-clicks a grid item.
' wParam:  Identifier of control
' lParam:  Address of GRIDITEMACTIVATE structure
' Return:  Return TRUE to prevent the default operation.

%GCN_ITEMRCLICK                             = %WM_USER + &H006B??
' Purpose: Sent to a grid control's parent window when the user clicks
'          the right mouse button on a grid item.
' wParam:  Identifier of control
' lParam:  Address of GRIDITEMACTIVATE structure
' Return:  Return TRUE to prevent the default operation.

%GCN_ITEMRDOUBLECLICK                       = %WM_USER + &H006C??
' Purpose: Sent to a grid control's parent window when the user double-clicks
'          the right mouse button on a grid item.
' wParam:  Identifier of control
' lParam:  Address of GRIDITEMACTIVATE structure
' Return:  Return TRUE to prevent the default operation.

%GCN_BEGINDRAG                              = %WM_USER + &H006D??
' Purpose: Sent by a grid control at the start of a drag operation on one of its items.
' wParam:  Identifier of control
' lParam:  Address of GRIDITEMACTIVATE structure
' Return:  TRUE to prevent the drag and drop operation.

%GCN_ENDDRAG                                = %WM_USER + &H006E??
' Purpose: Sent by a grid control when a drag and drop operation has ended.
' wParam:  Identifier of control
' lParam:  Address of GRIDITEMACTIVATE structure
' Return:  To prevent the dragged items from being placed, return TRUE.

%GCN_ITEMDRAW                               = %WM_USER + &H006F??
' Purpose: Sent by a grid control when an owner-drawn item is about to be drawn.
' wParam:  Identifier of control
' lParam:  Address of GRIDITEMDRAW structure
' Return:  N/A

%GCN_GETINFOTIP                             = %WM_USER + &H0070??
' Purpose: Sent to the grid control's parent window to retrieve informative text for an item.
' wParam:  Identifier of control
' lParam:  Address of a GRIDDISPINFO structure
' Return:  No return value.

%GCN_HOTITEMCHANGE                          = %WM_USER + &H0071??
' Purpose: Sent by the grid control when the mouse enters or leaves an item.
'          If the item is a flat button or header, it is highlighted when the mouse
'          passes over it.
' wParam:  Identifier of control
' lParam:  Address of GRIDHOTITEM structure
' Return:  Return FALSE to allow the default action, or non-zero to prevent it.

%GCN_SETFOCUS                               = %WM_USER + &H0072??
' Purpose: Sent when the control gains the input focus. The hWndFocus
'          member of the GRIDCONTROLFOCUS structure identifies the
'          window that has lost the keyboard focus (may be NULL).
' wParam:  Identifier of control
' lParam:  Address of GRIDCONTROLFOCUS structure
' Return:  The return value is ignored.

%GCN_KILLFOCUS                              = %WM_USER + &H0073??
' Purpose: Sent when the control is about to lose the input focus. The hWndFocus
'          member of the GRIDCONTROLFOCUS structure identifies the window
'          receives the keyboard focus (may be NULL).
' wParam:  Identifier of control
' lParam:  Address of GRIDCONTROLFOCUS structure
' Return:  The return value is ignored.

%GCN_KEYDOWN                                = %WM_USER + &H0074??
' Purpose: Sent to a grid control's parent window when a key has been pressed.
' wParam:  Identifier of control
' lParam:  Address of GRIDKEYDOWN structure
' Return:  Return TRUE to prevent the default action.

%GCN_FOCUSCHANGING                          = %WM_USER + &H0075??
' Purpose: Sent to a grid control's parent window when the focus is about to change.
'          This notification is sent when the change is initiated by the user.  It is not
'          sent when the focus is changed via the GM_SETFOCUSCELL message.
' wParam:  identifier of control
' lParam:  Address of GRIDITEMFOCUS structure
' Return:  Return TRUE to prevent the change, or FALSE to allow the change.

%GCN_FOCUSCHANGED                           = %WM_USER + &H0076??
' Purpose: Sent to a grid control' parent window when the focus has changed.
'          This notification is sent when the change is initiated by the user.  It is not
'          sent when the focus is changed via the GM_SetFocusCell message.
' wParam:  identifier of control
' lParam:  Address of GRIDITEMFOCUS structure
' Return:  No return value.

%GCN_ITEMCHANGED                            = %WM_USER + &H0077??
' Purpose: Sent to a grid control's parent window when one or more items have changed.
' wParam:  identifier of control
' lParam:  address of a GRIDDISPINFO structure
' Return:  No return value

%GCN_GETDISPINFO                            = %WM_USER + &H0078??
' Purpose: Sent to the grid control's parent window to request information needed to display an item.
' wParam:  Identifier of control
' lParam:  Address of a GRIDDISPINFO structure
' Return:  No return value

%GCN_DELETEITEM                             = %WM_USER + &H0079??
' Purpose: Sent to a grid control's parent window when one or more columns
'          or rows are about to be deleted.
' wParam:  Identifier of control
' lParam:  Address of a GRIDDISPINFO structure
' Return:  No return value

%GCN_INSERTITEM                             = %WM_USER + &H007A??
' Purpose: Sent to a grid control's parent window when one or more
'          columns or rows have been inserted.
' wParam:  Identifier of control
' lParam:  Address of a GRIDDISPINFO structure
' Return:  No return value

%GCN_CLEARCONTENTS                          = %WM_USER + &H007B??
' Purpose: Sent to a grid control's parent window when the contents of
'          one or more items have been cleared.
' wParam:  Identifier of control
' lParam:  Address of a GRIDDISPINFO structure
' Return:  No return value

%GCN_PASTE                                  = %WM_USER + &H007C??
' Purpose: Sent at the start of a paste operation in the grid control.
'          This notification is not sent during label editing.
' wParam:  Identifier of control
' lParam:  Address of NMHDR structure
' Return:  Return TRUE to prevent the paste operation, FALSE to allow it.

%GCN_MAXTEXTERROR                           = %WM_USER + &H007D??
' Purpose: Sent when the current text insertion has exceeded the maximum number of characters allowed for a masked cell.
' wParam:  Identifier of control
' lParam:  Address of a GRIDMAXTEXTERROR structure
' Return:  No return value

%GCN_VALIDATIONERROR                        = %WM_USER + &H007E??
' Purpose: Sent when the control receives invalid input, as determined by the input mask for a masked cell.
' wParam:  Identifier of control
' lParam:  Address of a GRIDVALIDATIONERROR structure
' Return:  No return value.

%GCN_ONCHARLABELEDIT                        = %WM_USER + &H007F??
' Purpose: Sent when the user types a character into the label-editing control.
' wParam:  Identifier of control
' lParam:  Address of a GRIDONCHAR structure
' Return:  No return value.

%GCN_ITEMEXPANDING                          = %WM_USER + &H0080??
' Purpose: Sent when a parent cell's list of child rows is about to expand or collapse.
' wParam:  Identifier of control
' lParam:  Address of a GRIDITEMEXPAND structure
' Return:  Return TRUE to prevent the list from expanding or collapsing.

%GCN_ITEMEXPANDED                           = %WM_USER + &H0081??
' Purpose: Sent when a parent cell's list of child rows has expanded or collapsed.
' wParam:  Identifier of control
' lParam:  Address of a GRIDITEMEXPAND structure
' Return:  The return value is ignored.

%GCN_NEEDITEMS                              = %WM_USER + &H0082??
' Purpose: Sent when the grid control needs to access child rows that have not yet been inserted for a parent cell.
' wParam:  Identifier of control
' lParam:  Address of a GRIDDISPINFO structure
' Return:  The return value is ignored.

%GCN_GETBUSYINFO                            = %WM_USER + &H0083??
' Purpose: Sent by the grid control when a parent cell that is about to be expanded
'          requests child rows.  This allows the control to display a message when a
'          large number of child rows are being inserted.
' wParam:  Identifier of control
' lParam:  Address of a GRIDDISPINFO structure
' Return:  The return value is ignored.

%GCN_MEASURELISTITEM                        = %WM_USER + &H0084??
' Purpose: Sent by the grid control when an owner-drawn drop-list is created.
' wParam:  Identifier of control
' lParam:  Address of a GRIDMEASURELISTITEM structure
' Return:  If an application processes this message, it should return TRUE.

%GCN_DRAWLISTITEM                           = %WM_USER + &H0085??
' Purpose: Sent by the grid control when an item is an owner-drawn drop-list is about to be drawn.
' wParam:  Identifier of control
' lParam:  Address of a GRIDDRAWLISTITEM structure
' Return:  If an application processes this message, it should return TRUE.

DECLARE FUNCTION InitGridControl LIB "GRID32.DLL" ALIAS "InitGridControl" () AS LONG

'###############################################################################
' END Phnx_Grid_Class32
'###############################################################################
